@@ -3,10 +3,11 @@
 """Common utilities."""
 
 import json
+from functools import lru_cache, singledispatch
 from typing import (
-    Set,
     TYPE_CHECKING,
     Any,
+    Dict,
     List,
     Mapping,
     Optional,
@@ -18,7 +19,11 @@ from typing import (
 )
 
 import jmespath
-from jmespath.parser import ParsedResult, Parser
+from django.db import DatabaseError
+from django.db.backends.base.base import BaseDatabaseWrapper
+from django.template import Context, Template
+from django.template.base import VariableNode
+from jmespath.parser import Parser
 from simpleeval import DEFAULT_FUNCTIONS, DEFAULT_OPERATORS, SimpleEval
 
 if TYPE_CHECKING:  # pragma: no cover
@@ -155,51 +160,12 @@ def stable_json(data: Union[dict, list]) -> str:
     )
 
 
-from string import Formatter
-
-
-class LenientFormatter(Formatter):
-    """A more lenient version of the default string formatter.
-
-    If a given variable was not specified in the format() kwargs, its value will be an empty string.
-
-    For example:
-
-    >>> formatter = LenientFormatter()
-    >>> test_string = "This variable was not provided: {not_provided}, but this one was: {provided}."
-    >>> formatter.format(test_string, provided="a value")
-    >>> 'This variable was not provided: , but this one was: a value.'
-    """
-
-    def get_value(
-        self, key: Union[int, str], args: Sequence[Any], kwargs: Mapping[str, Any]
-    ) -> Any:
-        """Return the value for the given key from the args or kwargs.
-
-        If the key does not exist in the args or kwargs, an empty string is
-        returned.
-
-        Args:
-            key: The name of the string variable for which to resolve the
-                value.
-            args: Positional arguments given to format().
-            kwargs: keyword arguments given to format().
-
-        Returns:
-            Any: The value for key, if given, otherwise an empty string.
-        """
-        try:
-            return super().get_value(key, args, kwargs)
-        except (KeyError, IndexError):
-            return ""
-
-
-def jp(expr: str, data: dict, default: Any = None) -> Any:
+def jp(expr: str, data: Any, default: Any = None) -> Any:
     """A shorthand helper for querying dicts with jmespath.
 
     Args:
         expr: The JMESPath expression.
-        data: The dict to query.
+        data: The data to query.
         default: The default value to return if the query returns None.
 
     Returns:
@@ -209,7 +175,8 @@ def jp(expr: str, data: dict, default: Any = None) -> Any:
     return default if result is None else result
 
 
-def get_expression_fields(jmespath_expression: str) -> Set[str]:
+@lru_cache(128)
+def get_expression_fields(jmespath_expression: str) -> Tuple[str, ...]:
     """Return a list of fields referenced in the given JMESPath expression.
 
     Args:
@@ -220,21 +187,147 @@ def get_expression_fields(jmespath_expression: str) -> Set[str]:
         Set[str]: A set containing the names of fields referenced in the
             expression.
     """
-    return _get_fields(Parser().parse(jmespath_expression).parsed)
+    return tuple(_get_fields(Parser().parse(jmespath_expression).parsed).keys())
 
 
-def _get_fields(node: str, ignore_fields: Sequence[str] = ("null",)) -> Set[str]:
-    referenced_fields = set()
+def _get_fields(
+    node: Dict[str, Any], ignore_fields: Sequence[str] = ("null",)
+) -> Dict[str, None]:
+    referenced_fields: Dict[str, None] = {}
 
     node_type = node["type"]
     children = node["children"]
     value = node.get("value")
 
     if node_type == "field" and value not in ignore_fields:
-        referenced_fields.add(value)
+        referenced_fields[str(value)] = None
     elif node_type == "subexpression":
         referenced_fields.update(_get_fields(children[0]))
     else:
-        referenced_fields.update(*(_get_fields(c) for c in children))
+        for child in children:
+            referenced_fields.update(_get_fields(child))
 
     return referenced_fields
+
+
+class RenderedString(str):
+    """A string with an attribute containing the render context.
+
+    The render_context property will be a dict containing the keys of
+    the variables used to render the string (variables in the original
+    context that were not used to render the string will be excluded).
+    """
+
+    render_context: Dict[str, Any]
+
+
+@singledispatch
+def interpolate(data: Any, context: Dict[str, Any]) -> Any:
+    """Interpolate the given data using DTL.
+
+    Handles (nested) dicts and lists and will interpolate strings containing DTL tokens.
+
+    Args:
+        data: The data to be interpolated.
+        context: The context with which to interpolate strings containing DTL tokens.
+
+    Returns:
+        Any: The interpolated data.
+    """
+    # The default implementation returns the data as-is.
+    return data
+
+
+@interpolate.register(str)
+def _interpolate_str(data: str, context: Dict[str, Any]) -> RenderedString:
+    """Handles interpolation of string values.
+
+    Interpolates strings using the default Django Template Language engine.
+
+    Args:
+        data: The data to be interpolated.
+        context: The context with which to interpolate strings containing DTL tokens.
+
+    Returns:
+        RenderedString: The rendered string with a render_context property
+            containing the variables used in rendering.
+    """
+    # Render the given string as a Django template with the given context.
+    template = Template(data)
+    template_context = Context(context, autoescape=False)
+    rendered_string = RenderedString(template.render(template_context))
+
+    # Extract a list of variables used in rendering and create a dict of
+    # them and their values to return attached to the rendered string.
+    rendered_string.render_context = {
+        var: template_context.get(var)
+        for var in frozenset(
+            v.filter_expression.var.lookups[0]
+            for v in template.nodelist
+            if isinstance(v, VariableNode)
+        )
+    }
+
+    return rendered_string
+
+
+@interpolate.register(dict)
+def _interpolate_dict(data: dict, context: Dict[str, Any]) -> dict:
+    """Handles interpolation of dict values.
+
+    Interpolates the values of the dict as appropriate (renders strings, or
+    recurses for list or dict values).
+
+    Args:
+        data: The data to be interpolated.
+        context: The context with which to interpolate strings containing DTL tokens.
+
+    Returns:
+        dict: The interpolated dict.
+    """
+    return {k: interpolate(v, context) for k, v in data.items()}
+
+
+@interpolate.register(list)
+def _interpolate_list(data: list, context: Dict[str, Any]) -> list:
+    """Handles interpolation of list values.
+
+    Interpolates the elements of the list as appropriate (renders strings, or
+    recurses for list or dict elements).
+
+    Args:
+        data: The list to be interpolated.
+        context: The context with which to interpolate strings containing DTL tokens.
+
+    Returns:
+        list: The interpolated list.
+    """
+    return [interpolate(v, context) for v in data]
+
+
+# Determine if the database support trigram similarity by checking for
+# the pg_trgm extension.
+def check_supports_pg_trgm(connection: BaseDatabaseWrapper) -> bool:
+    """Determine if trigram similarity support is present.
+
+    Attempts to run a query against the extensions table in PostgreSQL to
+    look for the pg_trgm extension. Failures assume a lack of support.
+
+    Args:
+        connection: The database connection.
+
+    Returns:
+        bool: True if trigram support is present.
+    """
+    try:
+        with connection.cursor() as cursor:
+            cursor.execute(
+                """
+                SELECT 1
+                FROM pg_extension
+                WHERE extname = 'pg_trgm' LIMIT 1;
+            """
+            )
+            return bool(cursor.fetchone())
+    except DatabaseError:
+        return False
